#
#--------------------------------------------------------------------
#   Include
#--------------------------------------------------------------------
import math as M
#
#--------------------------------------------------------------------
#   Helper
#--------------------------------------------------------------------
def Error(text):
    print('Error: ' + text + '!')
    return
#
def PrintMatrix(title, name, m):
    if (None == m):
        Error('Zero Matrix')
        return
    NR = len(m)
    NC = len(m[0])
    print('{0}({1})[{2}|{3}]:'.format(title, name, NR, NC))
    for IR in range(0, NR):
        for IC in range(0, NC):
            print('%+7.2f' % (m[IR][IC]), end='')
        print('')
    return
#
def PrintSolution(title, name, m):
    if (None == m):
        Error('Zero Matrix')
        return
    NR = len(m)
    NC = len(m[0])
    if (NR != NC - 1):
        Error('Invalid Matrix-Dimension')
        return
    print('{0}[{1}|{2}]:'.format(title, NR, NC))
    for IR in range(0, NR):
        print('%s[%d]=%+6.2f' % (name, IR, m[IR][NC - 1]))
    return
#
#--------------------------------------------------------------------
#   Solver
#--------------------------------------------------------------------
def SolverNormalizeMatrix(m):
    if (None == m):
        Error('Zero Matrix')
        return []
    NR = len(m)
    NC = len(m[0])
    for IR in range(0, NR):
        D = m[IR][IR]
        if (0 == D):
            Error('Division by zero')
            return []
        for IC in range(0, NC):
            m[IR][IC] = m[IR][IC] / D
    return m
#
def SolverLowerElemination(m):
    NR = len(m)
    NC = len(m[0])
    for IT in range(0, NR):
        for IR in range(1 + IT, NR):
            F = m[IR][IT] / m[IT][IT]
            for IC in range(0, NC):
                m[IR][IC] = m[IR][IC] - F * m[IT][IC]             
    return m
#
def SolverGaussElemination(m):
    NR = len(m)
    NC = len(m[0])
    for IT in range(0, NR):
        for IR in range(0, IT):
            F = m[IR][IT] / m[IT][IT]
            for IC in range(0, NC):
                m[IR][IC] = m[IR][IC] - F * m[IT][IC] 
        for IR in range(1 + IT, NR):
            F = m[IR][IT] / m[IT][IT]
            for IC in range(0, NC):
                m[IR][IC] = m[IR][IC] - F * m[IT][IC] 
    return m
#
#--------------------------------------------------------------------
#   Main
#--------------------------------------------------------------------
if ('__main__' == __name__):
    print('*** GaussElemination: begin')   
    #
    M = [[-1,  1,  1,  0], 
         [ 1, -3, -2,  5], 
         [ 5,  1,  4,  3]]
    PrintMatrix('Matrix', 'Source', M)
    M = SolverGaussElemination(M)
    PrintMatrix('Matrix', 'GaussEleminated', M)
    M = SolverNormalizeMatrix(M)
    PrintMatrix('Matrix', 'Normalized', M)
    PrintSolution('GaussSolution', 'X', M)
    #
    print('*** GaussElemination: end')
    












    